from typing import Optional, Type
from langchain_openai_api_bridge.assistant.adapter.container import (
    register_assistant_adapter,
)
from langchain_openai_api_bridge.assistant.assistant_message_service import (
    AssistantMessageService,
)
from langchain_openai_api_bridge.assistant.assistant_run_service import (
    AssistantRunService,
)
from langchain_openai_api_bridge.assistant.assistant_thread_service import (
    AssistantThreadService,
)
from langchain_openai_api_bridge.assistant.repository.message_repository import (
    MessageRepository,
)
from langchain_openai_api_bridge.assistant.repository.run_repository import (
    RunRepository,
)
from langchain_openai_api_bridge.assistant.repository.thread_repository import (
    ThreadRepository,
)
from langchain_openai_api_bridge.core.agent_factory import AgentFactory
from langchain_openai_api_bridge.core.utils.di_container import DIContainer


class AssistantApp:
    def __init__(
        self,
        thread_repository_type: Type[ThreadRepository],
        message_repository_type: Type[MessageRepository],
        run_repository: Type[RunRepository],
        agent_factory: Type[AgentFactory],
        system_fingerprint: Optional[str] = "",
    ):
        self.container = DIContainer()
        self.system_fingerprint = system_fingerprint

        register_assistant_adapter(self.container)

        self.container.register(AssistantThreadService)
        self.container.register(AssistantMessageService)
        self.container.register(AssistantRunService)

        self.container.register(
            ThreadRepository, to=thread_repository_type, singleton=True
        )
        self.container.register(
            MessageRepository, to=message_repository_type, singleton=True
        )
        self.container.register(RunRepository, to=run_repository, singleton=True)
        self.container.register(AgentFactory, to=agent_factory)



from fastapi import APIRouter, FastAPI

from langchain_openai_api_bridge.assistant.assistant_app import AssistantApp
from langchain_openai_api_bridge.fastapi.add_assistant_routes import (
    create_open_ai_compatible_assistant_router,
)


def include_assistant(app: FastAPI, assistant_app: AssistantApp, prefix: str = ""):
    assistant_router = create_open_ai_compatible_assistant_router(
        assistant_app=assistant_app
    )
    open_ai_router = APIRouter(prefix=f"{prefix}/openai/v1")
    open_ai_router.include_router(assistant_router)

    app.include_router(open_ai_router)



from fastapi import APIRouter, FastAPI

from langchain_openai_api_bridge.assistant.assistant_app import AssistantApp

from langchain_openai_api_bridge.fastapi.add_chat_completions_agent_routes import (
    create_open_ai_compatible_chat_completion_router,
)


def include_chat_completion(
    app: FastAPI, assistant_app: AssistantApp, prefix: str = ""
):
    chat_completion_routes = create_open_ai_compatible_chat_completion_router(
        assistant_app=assistant_app
    )
    open_ai_router = APIRouter(prefix=f"{prefix}/openai/v1")
    open_ai_router.include_router(chat_completion_routes)

    app.include_router(open_ai_router)



from .assistant_route_builder import include_assistant
from .add_assistant_routes import create_open_ai_compatible_assistant_router
from .chat_completion_route_builder import include_chat_completion


__all__ = [
    "include_assistant",
    "create_open_ai_compatible_assistant_router",
    "include_chat_completion",
]



from openai.types.beta import Thread, ThreadDeleted
from abc import ABC, abstractmethod


class ThreadRepository(ABC):

    def __init__(
        self,
    ) -> None:
        pass

    @abstractmethod
    def create(
        self,
    ) -> Thread:
        # client.beta.threads.create(messages)
        pass

    @abstractmethod
    def retreive(self, thread_id: str) -> Thread:
        # client.beta.threads.retrieve()
        pass

    @abstractmethod
    def delete(
        self,
        thread_id: str,
    ) -> ThreadDeleted:
        # client.beta.threads.delete()
        pass



import time
import uuid
from .thread_repository import ThreadRepository
from typing import Optional
from openai.types.beta import Thread, ThreadDeleted


class InMemoryThreadRepository(ThreadRepository):
    def __init__(self):
        self.threads: dict[str, Thread] = {}

    def create(self, metadata: Optional[object] = None) -> Thread:
        thread_id = str(uuid.uuid4())
        thread = self.__create_thread(thread_id=thread_id, metadata=metadata)
        self.threads[thread_id] = thread

        return self.retreive(thread_id)

    def retreive(self, thread_id: str) -> Thread | None:
        result = self.threads.get(thread_id)
        if result is None:
            return None

        return result.copy(deep=True)

    def delete(
        self,
        thread_id: str,
    ) -> ThreadDeleted:
        thread = self.retreive(thread_id)
        if thread is None:
            return None
        del self.threads[thread_id]
        return self.__create_thread_deleted(thread_id=thread_id)

    @staticmethod
    def __create_thread(thread_id: str, metadata: Optional[object] = None) -> Thread:
        return Thread(
            id=thread_id, object="thread", created_at=time.time(), metadata=metadata
        )

    @staticmethod
    def __create_thread_deleted(thread_id: str) -> ThreadDeleted:
        return ThreadDeleted(id=thread_id, object="thread.deleted", deleted=True)
